﻿package code {
	import flash.display.MovieClip;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	import libs.PathFinder.*;
	
	public class Demo extends MovieClip {
		private const gridSize = 30;
		private const gridHalf = gridSize / 2;
		private const gridXOffset = 0;
		private const gridYOffset = 0;
		private const numCols = 7;
		private const numRows = 6;
		
		private var dragX;
		private var dragY;
		
		private var startTile:MovieClip;
		private var endTile:MovieClip;
		
		private var level:Array;
		private var walls:Array;
		private var hitboxes:Array;
		
		private var moves:Array;
		private var moveSpots:Array;
		
	//Constructor
		public function Demo() {
			this.gotoAndStop(1);
			this.LoadLevel();
		}
		
	//setup the level
		private function LoadLevel():void {
			var coords:Array;
			
			//level on screen as 2D array (0 walkable, 1 obstacle)
			level = [
				[0, 0, 0, 0, 1, 0, 0, 0],
				[1, 1, 1, 0, 1, 0, 1, 0],
				[0, 0, 0, 0, 1, 0, 1, 0],
				[0, 0, 0, 0, 1, 0, 1, 0],
				[0, 0, 1, 1, 1, 0, 1, 0],
				[0, 0, 0, 0, 1, 0, 1, 0],
				[0, 0, 0, 0, 0, 0, 1, 0]];
			
			this.PlaceWalls();
			this.PlaceHitboxes();
			
			startTile = new Start();
			coords = this.Grid2Pixel(2, 2);
			startTile.x = coords[0];
			startTile.y = coords[1];
			this.addChild(startTile);
			startTile.buttonMode = true;
			startTile.mouseChildren = false;
			startTile.addEventListener(MouseEvent.MOUSE_DOWN, DragObject);
			
			endTile = new End();
			coords = this.Grid2Pixel(7, 6);
			endTile.x = coords[0];
			endTile.y = coords[1];
			this.addChild(endTile);
			endTile.buttonMode = true;
			endTile.mouseChildren = false;
			endTile.addEventListener(MouseEvent.MOUSE_DOWN, DragObject);
			
			//initial path
			movespots = new Array();
			this.FindPath();
		}
		
	//draw the wall visuals
		private function PlaceWalls():void {
			walls = new Array();
			
			//place on all ones in level array
			for (var i = 0; i <= numRows; i++) {
				for (var j = 0; j <= numCols; j++) {
					if (level[i][j] == 1) {
						var wall:MovieClip = new Wall();
						var coords:Array = this.Grid2Pixel(j, i);
						wall.x = coords[0];
						wall.y = coords[1];
						this.addChild(wall);
						walls.push(wall);
					}
				}
			}
		}
		
	//setup invisible grid of hitboxes
		private function PlaceHitboxes():void {
			hitboxes = new Array();
			
			//place on all zeros in level array
			for (var i = 0; i <= numRows; i++) {
				for (var j = 0; j <= numCols; j++) {
					if (level[i][j] == 0) {
						var hitbox:MovieClip = new HitBox();
						var coords:Array = this.Grid2Pixel(j, i);
						hitbox.x = coords[0];
						hitbox.y = coords[1];
						hitboxes.push(hitbox);
					}
				}
			}
		}
		
	//run the pathfinding algorithm
		private function FindPath():void {
			var coords1:Array = this.Pixel2Grid(startTile.x, startTile.y);
			var coords2:Array = this.Pixel2Grid(endTile.x, endTile.y);
			moves = PathFinder.go(coords1[0], coords1[1], coords2[0], coords2[1], level);
			this.PlaceMoves();
		}
		
	//draw move spots after pathfinding
		private function PlaceMoves():void {
			var i = 0;
			for each (var move in moves) {
				i++;
				var spot:MovieClip = new MoveSpot();
				var coords:Array = this.Grid2Pixel(move.x, move.y);
				spot.x = coords[0];
				spot.y = coords[1];
				spot.num.text = i;
				this.addChild(spot);
				movespots.push(spot);
			}
			
			//to actually move along path
			//move toward first spot on path until
			//if (Point.distance(new Point(this.x,this.y),target) < 2) {this.ChooseTarget();}		//reached target
		}
		
	//drag and drop functionality
		private function DragObject(e:MouseEvent):void {
			e.target.startDrag();
			e.target.addEventListener(MouseEvent.MOUSE_UP, DropObject);
			
			//initial position
			dragX = e.target.x;
			dragY = e.target.y;
			
			//remove path
			for each (var spot in movespots) {
				this.removeChild(spot);
			}
			movespots.length = 0;
		}
		private function DropObject(e:MouseEvent):void {
			e.target.stopDrag();
			e.target.removeEventListener(MouseEvent.MOUSE_UP, DropObject);
			
			//check hitboxes
			var missed = true;
			for each (var hitbox in hitboxes) {
				if (e.target.hitTestObject(hitbox)) {
					e.target.x = hitbox.x;
					e.target.y = hitbox.y;
					missed = false;
				}
			}
			if (missed) {
				e.target.x = dragX;
				e.target.y = dragY;
			}
			
			this.FindPath();
		}
		
	//conversion between pixel coordinates and grid coordinates
		public function Pixel2Grid(xPos, yPos):Array {
			var coords:Array = new Array();
			var col;
			var row;
			
			//determine grid space (start from 0)
			col = int((xPos - gridXOffset - gridHalf) / gridSize);	
			row = int((yPos - gridYOffset - gridHalf) / gridSize);
			
			//fill array
			coords.push(col);
			coords.push(row);
			return coords;
		}
		public function Grid2Pixel(col, row):Array {
			var coords:Array = new Array();
			var xPos;
			var yPos;
			
			xPos = col * gridSize + gridXOffset + gridHalf;
			yPos = row * gridSize + gridYOffset + gridHalf;
			
			//fill array
			coords.push(xPos);
			coords.push(yPos);
			return coords;
		}
		
//end of program
	}
}